
--********************************
-- Sources tables
--********************************
CREATE TABLE "sources" (
   "id_source" serial PRIMARY KEY,
   "public_key"  text);


CREATE TABLE "sources_informations" (
   "id_information" serial PRIMARY KEY,
   "id_source" int4 NOT NULL 
               CONSTRAINT si_id_source_fkey 
               REFERENCES sources (id_source),
   "description" text NOT NULL,
   "value" text,

   UNIQUE ("id_source", "description")
);




--********************************
-- Agents tables
--********************************
CREATE TABLE "agents" (
   "id_agent" serial PRIMARY KEY,
   "id_source" int4 NOT NULL 
               CONSTRAINT agents_id_source_fkey
               REFERENCES sources (id_source),
   "number" int4 NOT NULL,
   "version" int4 NOT NULL,
   "activationdate" datetime NOT NULL,
   "information" text,
   "lastconnectiondate" datetime,

   UNIQUE ("id_source", "number"), 
   UNIQUE ("number", "version", "activationdate")
);

--********************************
-- Alarms tables
--********************************
CREATE TABLE "alarms_levels" (
   "id_alarm_level" serial PRIMARY KEY,
   "level" text,
   "localisation" text,
   "image" text,

   UNIQUE ("level")
);

COPY "alarms_levels" FROM stdin;
1	Info	Information
2	Warning	Warning
3	Error	Erreur
\.


CREATE TABLE "agents_alarms_descriptions" (
   "id_alarm_description" serial PRIMARY KEY,
   "id_agent" int4 NOT NULL
              CONSTRAINT aad_id_agent_fkey 
              REFERENCES agents (id_agent),
   "alarm_number" int4 NOT NULL,
   "id_alarm_level" int4 
                    CONSTRAINT aad_id_aalarm_level_fkey 
                    REFERENCES alarms_levels (id_alarm_level),
   "message" text,

   UNIQUE ("id_agent", "alarm_number")
);

CREATE INDEX "aad_ida_key"
   on "agents_alarms_descriptions"
   using btree ( "id_agent" );


CREATE TABLE "complex_alarms_descriptions" (
   "last_checked_data_date" datetime DEFAULT 'epoch' NOT NULL
) inherits ("agents_alarms_descriptions");



CREATE TABLE "alarms" (
   "id_alarm" serial PRIMARY KEY,
   "id_alarm_description" int4 NOT NULL 
                          CONSTRAINT alarms_id_alrm_desc_fkey 
                          REFERENCES agents_alarms_descriptions (id_alarm_description),
   "storage_date" datetime NOT NULL,
   "release_date" datetime NOT NULL,
   "acquittal_date" datetime,

   UNIQUE ("id_alarm_description", "release_date")
);

CREATE INDEX "am_idad_acquit_key"
   on "alarms"
   using btree ( "id_alarm_description", "acquittal_date");



--********************************
-- Agents description tables
--********************************

CREATE TABLE "data_units" (
   "id_data_unit" serial PRIMARY KEY,
   "unit" text NOT NULL,
   "localisation" text,

   UNIQUE ("unit")   
);

COPY "data_units" FROM stdin;
1	bytes	octets
2	kilobytes	kilo-octets
3	percentage	%
4	none
\.


CREATE TABLE "data_types" (
   "id_data_type" serial PRIMARY KEY,
   "type" text NOT NULL,

   UNIQUE ("type")
);

COPY "data_types" FROM stdin;
1	Integer
2	Real
3	Text
4	Date
5	Time
6	DateTime
\.

CREATE TABLE "agents_data_descriptions" (
   "id_data_description" serial PRIMARY KEY,
   "id_agent" int4 NOT NULL 
              CONSTRAINT add_id_agent_fkey
              REFERENCES agents (id_agent),
   "number" int4 NOT NULL,
   "type" int4 NOT NULL 
          CONSTRAINT add_type_fkey
          REFERENCES data_types (id_data_type),
   "unit" int4 NOT NULL 
          CONSTRAINT add_unit_fkey
          REFERENCES data_units (id_data_unit),
   "description" text,
   "ancestor" int4 DEFAULT NULL,

   UNIQUE ("id_agent", "number"),
   UNIQUE ("id_agent", "description")
);

CREATE INDEX "add_ida_key"
   on "agents_data_descriptions"
   using btree ( "id_agent" );


--********************************
-- Data tables
--********************************

CREATE TABLE data (
        "id_data" serial PRIMARY KEY,
        "id_data_description" int4 NOT NULL 
                              CONSTRAINT data_id_data_desc_fkey
                              REFERENCES agents_data_descriptions (id_data_description),
        "storage_date" datetime NOT NULL,
        "collection_date" datetime NOT NULL,

        unique(id_data_description, storage_date, collection_date),
        unique(id_data_description, collection_date)
);

CREATE INDEX "d_iddd_key"
   on "data"
   using btree ( "id_data_description" );


CREATE INDEX "d_coll_date_key"
   on "data"
   using btree ( "collection_date" );



CREATE TABLE "integer_values" (
   "value" int4
) inherits (data);

CREATE TABLE "real_values" (
        value   float4
) inherits (data);

CREATE TABLE "text_values" (
   "value" text
) inherits (data);


CREATE TABLE "date_values" (
   "value" date
) inherits (data);

CREATE TABLE "datetime_values" (
   "value" datetime
) inherits (data);


CREATE TABLE "time_values" (
   "value" time
) inherits (data);




--************************************************--
--   INDEXES, FOREIGN KEYS AND UNIQUE CONSTRAINT 
--   ARE NOT YET INHERITED. 
--   FOLLOWING REQUEST SHOULD BE REMOVED 
--   IN FUTURE RELEASE
--************************************************--

ALTER TABLE integer_values ADD CONSTRAINT integer_value_fkey 
      FOREIGN KEY (id_data_description) 
      REFERENCES agents_data_descriptions (id_data_description);

ALTER TABLE real_values ADD CONSTRAINT real_value_fkey 
      FOREIGN KEY (id_data_description) 
      REFERENCES agents_data_descriptions (id_data_description);

ALTER TABLE date_values ADD CONSTRAINT date_value_fkey 
      FOREIGN KEY (id_data_description) 
      REFERENCES agents_data_descriptions (id_data_description);

ALTER TABLE text_values ADD CONSTRAINT text_value_fkey 
      FOREIGN KEY (id_data_description) 
      REFERENCES agents_data_descriptions (id_data_description);

ALTER TABLE datetime_values ADD CONSTRAINT datetime_value_fkey 
      FOREIGN KEY (id_data_description) 
      REFERENCES agents_data_descriptions (id_data_description);

ALTER TABLE time_values ADD CONSTRAINT time_value_fkey 
      FOREIGN KEY (id_data_description) 
      REFERENCES agents_data_descriptions (id_data_description);


CREATE UNIQUE INDEX "int_val_pkey"
   on "integer_values"
   using btree ("id_data");

CREATE UNIQUE INDEX "real_val_pkey"
   on "real_values"
   using btree ("id_data");

CREATE UNIQUE INDEX "date_val_pkey"
   on "date_values"
   using btree ("id_data");

CREATE UNIQUE INDEX "text_val_pkey"
   on "text_values"
   using btree ("id_data");

CREATE UNIQUE INDEX "datetime_val_pkey"
   on "datetime_values"
   using btree ("id_data");

CREATE UNIQUE INDEX "time_val_pkey"
   on "time_values"
   using btree ("id_data");





CREATE UNIQUE INDEX "int_id_data_coll_key"
   on "integer_values"
   using btree (id_data_description, collection_date);

CREATE UNIQUE INDEX "real_id_data_coll_key"
   on "real_values"
   using btree (id_data_description, collection_date);


CREATE INDEX "int_iddd_key"
   on "integer_values"
   using btree ( "id_data_description" );

CREATE INDEX "real_iddd_key"
   on "real_values"
   using btree ( "id_data_description" );


CREATE INDEX "int_coll_date_key"
   on "integer_values"
   using btree ( "collection_date" );

CREATE INDEX "real_coll_date_key"
   on "real_values"
   using btree ( "collection_date" );


ALTER TABLE complex_alarms_descriptions ADD CONSTRAINT cad_id_agent_fkey
      FOREIGN KEY (id_agent) 
      REFERENCES agents (id_agent);

ALTER TABLE complex_alarms_descriptions ADD CONSTRAINT cad_id_alarm_level_fkey
      FOREIGN KEY (id_alarm_level) 
      REFERENCES alarms_levels (id_alarm_level);


CREATE UNIQUE INDEX "cad_pkey"
   on "complex_alarms_descriptions"
   using btree ("id_alarm_description");

CREATE UNIQUE INDEX "cad_ida_aln_key"
   on "complex_alarms_descriptions"
   using btree ("id_agent", "alarm_number");

CREATE INDEX "cad_ida_key"
   on "complex_alarms_descriptions"
   using btree ( "id_agent" );



--********************************
-- Grant access on table
--********************************
GRANT all
   ON agents, agents_alarms_descriptions, agents_data_descriptions,
      alarms, alarms_levels, data, data_types, data_units, date_values,
      datetime_values, integer_values, real_values, sources,
      sources_informations, text_values, time_values,
      complex_alarms_descriptions
   TO logtrend;


--********************************
-- Grant access on sequences
--********************************
GRANT all 
   ON agents_alarms_id_alarm_desc_seq, agents_data_d_id_data_descr_seq,
      agents_id_agent_seq, alarms_id_alarm_seq, alarms_levels_id_alarm_leve_seq,
      data_id_data_seq, data_types_id_data_type_seq, data_units_id_data_unit_seq,
      sources_id_source_seq, sources_infor_id_informatio_seq
   TO logtrend;
